/*
* %Z%%W% %I%
*
===========================================================================
* Licensed Materials - Property of IBM
* "Restricted Materials of IBM"
* (C) Copyright IBM Corp. 2005. All Rights Reserved
*
===========================================================================
*/
package com.ibm.jzos.sample;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Iterator;
import java.util.Properties;

import com.ibm.jzos.Exec;
import com.ibm.jzos.RcException;
import com.ibm.jzos.ZUtil;

/**
 * Sample program which reads all sysout data for a MvsJob (jobname and jobid),
 * and writes the output to a specified Writer. 
 * <p>
 * The class relies on the sample Rexx script "jobOutput", spawned as a child process
 * via the {@link com.ibm.jzos.Exec} class.
 * <p/>
 @since 2.1.0
 */
public class MvsJobOutput {

	
	public static final String JOB_OUTPUT_CMD = "jobOutput";
	
	/**
	 * A sample main method that writes sysout output for 
	 * a job to System.out (STDOUT).
	 * <p>
	 * The first argument is the jobname, and the second argument
	 * is the jobid (JOBnnnnn).
	 */
	public static void main(String[] args) throws IOException {
		
		if (args.length < 2 ) {
			throw new IllegalArgumentException("Missing arguments: jobname jobid ");
		}
		
		MvsJob mvsJob = new MvsJob(args[0], args[1]);
		
		// print out the status of the job.
		// this will throw an exception if there is no such job
		System.out.println("JOB " + mvsJob + " " + mvsJob.getStatus());

		BufferedWriter writer = new BufferedWriter(
				new OutputStreamWriter(
						System.out));

		MvsJobOutput.writeJobOutput(mvsJob, writer);
			
	}

	
	/**
	 * Writes all of the output for a given job to a writer.
	 * Note: this method flushes the writer, but does not close it; 
	 * it is the callers' responsibility to close the writer.
	 */
	public static void writeJobOutput(MvsJob mvsJob, Writer writer) throws IOException {
		
		Exec exec = new Exec(getJobOutputCommand(mvsJob), getEnvironment());
		exec.run();
		
		try {
			String line;
			while ((line = exec.readLine()) != null) {
				writer.write(line);
				writer.write('\n');
			}; 
			writer.flush();

		} finally {
			int rc = exec.getReturnCode();
			if (rc != 0) {
				throw new RcException("Rexx 'jobOutput' process failed", rc);
			}
		}
	}
	
	/*
	 * Return the command to be executed via Runtime.exec().  This is
	 * the Rexx script 'submitJob'.  By default, this script needs to be present
	 * in the current PATH.  However, if the System variable jzos.script.path 
	 * is defined, it will be used to prefix 'submitJob'.
	 */
	protected static String getJobOutputCommand(MvsJob mvsJob) {
		String cmdPath = System.getProperty("jzos.script.path", "");
		if (cmdPath.length() > 0 && !cmdPath.endsWith("/")) {
			cmdPath = cmdPath + "/";
		}
		return cmdPath + JOB_OUTPUT_CMD 
				+ " " 
				+ mvsJob.getJobname() 
				+ " "
				+ mvsJob.getJobid();
	}

	/*
	 * Return the environment to use for the child process.  This is
	 * the current environment with _BPX_SHAREAS and _BPX_SPAWN_SCRIPT set to "YES"
	 * so that the child process will execute in the same address space.
	 */
	protected static String[] getEnvironment() {
		Properties p = ZUtil.getEnvironment();
		p.put("_BPX_SHAREAS", "YES");
		p.put("_BPX_SPAWN_SCRIPT", "YES");
		String[] environ = new String[p.size()];
		int i=0;
	    for (Iterator iter = p.keySet().iterator(); iter.hasNext();) {
	    	String key = (String)iter.next();
	    	environ[i++] = key + "=" + p.getProperty(key);
	    }
	    return environ;
	}
}
