/*
* %Z%%W% %I%
*
===========================================================================
* Licensed Materials - Property of IBM
* "Restricted Materials of IBM"
* (C) Copyright IBM Corp. 2005. All Rights Reserved
*
===========================================================================
*/
package com.ibm.jzos.sample;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Iterator;
import java.util.Properties;
import java.util.StringTokenizer;

import com.ibm.jzos.Exec;
import com.ibm.jzos.FileFactory;
import com.ibm.jzos.RcException;
import com.ibm.jzos.ZUtil;

/**
 * Sample program which submits a job to the internal reader, 
 * returning a {@link MvsJob}  object containing the jobname and jobid.
 * <p>
 * The class relies on sample Rexx scripts spawned as a child process
 * via the {@link com.ibm.jzos.Exec} class.
 * <p>
 * The JZOS ZFile class can be used to submit JCL if you use {@link com.ibm.jzos.ZFile#bpxwdyn(String)}
 * to allocate a DD to "SYSOUT(A) WRITER(INTRDR)", but this mechanism does not have the
 * ability to return the submitted job id (JOBnnnnn).  Fortunately, the Rexx "submit()" function 
 * has the ability to submit JCL and get the jobid back, so this example shows how to attach
 * a sample USS Rexx script ("submitJob") as a child process and send it JCL on its stdin handle. 
 * <p>
 * In order for JZOS to find the sample REXX script, the System property 
 * <strong>jzos.script.path</strong> must be set to point to the sample scripts directory.  See the
 * Installation Guide for details.
 * <p>
 * Similarly, getting status for an executing job requires an APF authorized program interface
 * to the subsystem api.  The TSO "STATUS" command can be executed via the Rexx "TSO" command
 * processor to obtain this information.  See the sample "jobStatus" Rexx script.   
 *
 * <p/>
 * @since 2.1.0
 */
public class MvsJobSubmitter {

	public static final long ONE_SECOND = 1000;
	public static final long TWO_SECONDS = 2 * ONE_SECOND;
	public static final long ONE_MINUTE = 60 * ONE_SECOND;
	
	public static final String SUBMIT_JOB_CMD = "submitJob";
	public static final String JOB_STATUS_CMD = "jobStatus";
	
	Exec exec;
	BufferedWriter intrdrWriter;
	
	/**
	 * A sample main method that submits JCL read from a file and then 
	 * polls its status (for up to a minute) until it is complete.
	 * <p>
	 * The first argument to main can be a Unix file/path name
	 * or a "//dataset.name".  The name is given directory to 
	 * the {@link com.ibm.jzos.FileFactory#newBufferedReader(String)} method
	 * to open the JCL file to submit.
	 */
	public static void main(String[] args) throws IOException {
		
		if (args.length < 1 ) {
			throw new IllegalArgumentException("Missing main argument: filename ");
		}
		
		MvsJobSubmitter submitter = new MvsJobSubmitter();
		
		// copy the file to the InternalReaderWriter (and close that writer)
		copyFile(args[0], 
				submitter.getInternalReaderWriter());
			
		MvsJob job = submitter.submitJob();
			
		long begin = System.currentTimeMillis();
		boolean completed = false;
		do {
			String status = job.getStatus();
			System.out.println("" + job + " " + status);
			
			try {
				Thread.sleep(TWO_SECONDS);
			} catch (InterruptedException ignore) {}

			completed = !status.startsWith("WAITING")
					&& !status.startsWith("EXECUTING");

		} while (!completed && System.currentTimeMillis() - begin < ONE_MINUTE );
			
	}

	
	/**
	 * Construct an instance, which causes the "submitJob" Rexx script to 
	 * be started to receive our JCL.  
	 */
	public MvsJobSubmitter() throws IOException {
		executeSubmit();  // starts the child process
	}
	
	/**
	 * Answers a writer that can be used by the client to 
	 * send JCL to the Rexx submitJob process.
	 * <note>
	 * Clients should either flush() or close() this Writer
	 * to ensure that buffered data is actually sent to the child process
     * <p>
	 * @return Writer
	 * @throws IOException
	 */
	public Writer getInternalReaderWriter() throws IOException {

		if (intrdrWriter == null) {
			intrdrWriter = exec.getStdinWriter();
		}
		return intrdrWriter;
	}
	
	/**
	 * Answers an MvsJob object which is a simple bean holding the
	 * submitted jobname and userid.  The method should be called
	 * once after writing jcl to getInternalReaderWriter() and closing
	 * the writer.
	 * <p>
	 * @return MvsJob if submitted successfully, otherwise an exception is thrown
	 * 
	 * @throws IOException if there is an error communicating with the child Rexx process
	 * or if the child process had an error submitting the process.
	 */
	public MvsJob submitJob() throws IOException {
		if (exec == null) { 
			 // can only do this once!
			throw new IllegalStateException("Child process already stopped?"); 
		}
		MvsJob mvsjob = null;
        String line;
        String firstLine = null;
        while ((line = exec.readLine()) != null) {
        	if (firstLine == null) firstLine = line;
        	if (mvsjob == null) {
        		StringTokenizer tok = new StringTokenizer(line);
        		if (tok.countTokens() == 3) {
        			if (tok.nextToken().equalsIgnoreCase("Submitted:")) {
        				String jobname = tok.nextToken();
        				String jobid   = tok.nextToken();
        				mvsjob = new MvsJob(jobname, jobid);
        			}
        		}
        	}
        }
        int rc = exec.getReturnCode();  // wait for process completion
        exec = null;
        if (mvsjob == null) rc = 3; // somehow we got a good return code with invalid output
        if (rc != 0) {
        	throw new RcException("Rexx 'submitJob' process failed: " + firstLine, rc);
        }
        return mvsjob;
	}
	
	/**
	 * Returns the status for a job using the TSO "status" command, 
	 * which is invoked via the "jobStatus" REXX USS script.
	 * 
	 * @return String the TSO "STATUS" command status
	 * @throws IOException if there was an error communicating with the child REXX script process
	 */
	public static String getJobStatus(MvsJob job) throws IOException {
		Exec exec = new Exec(getStatusCommand(job), getEnvironment());
		exec.run();
		
		String line = exec.readLine();
		if (line == null) throw new IOException("No output from jobStatus child process");

		while (exec.readLine() != null) {};  // slurp any other output
		int rc = exec.getReturnCode();
		if (rc != 0) {
			throw new RcException("Rexx 'jobStatus' process failed: " + line, rc);
		}
		
		StringTokenizer tok = new StringTokenizer(line);
		if (tok.countTokens() < 3 ) {
			throw new IOException("Invalid output from jobStatus child process: " + line);
		}
		String next = tok.nextToken();
		if (next.startsWith("IKJ")) {
			next = tok.nextToken();  // skip over msg id
		}
		if (!next.equalsIgnoreCase("JOB")) {
			throw new IOException("Invalid output from jobStatus child process: " + line);
		}
		
		tok.nextToken();  // skip jobname(jobid)
	
		String answer = "";
		while (tok.hasMoreTokens()) {
			answer += tok.nextToken();  // concat remaining words
			if (tok.hasMoreTokens()) answer += " ";
		}
		
		return answer;
	}	
	
	/**
	 * Start the "submitJob" child process.  
	 */
	protected void executeSubmit() throws IOException {
		exec = new Exec(getSubmitCommand(), getEnvironment());
		exec.run();
	}
	
	/*
	 * Return the command to be executed via Runtime.exec().  This is
	 * the Rexx script 'submitJob'.  By default, this script needs to be present
	 * in the current PATH.  However, if the System variable jzos.script.path 
	 * is defined, it will be used to prefix 'submitJob'.
	 */
	protected static String getSubmitCommand() {
		String cmdPath = System.getProperty("jzos.script.path", "");
		if (cmdPath.length() > 0 && !cmdPath.endsWith("/")) {
			cmdPath = cmdPath + "/";
		}
		return cmdPath + SUBMIT_JOB_CMD;
	}

	/*
	 * Return the command to be executed via Runtime.exec().  This is
	 * the Rexx script 'jobStatus' followed by the jobnamed and jobid.  
	 * By default, this script needs to be present
	 * in the current PATH.  However, if the System variable jzos.script.path 
	 * is defined, it will be used to prefix 'jobStatus'.
	 */
	protected static String getStatusCommand(MvsJob job) {
		String cmdPath = System.getProperty("jzos.script.path", "");
		if (cmdPath.length() > 0 && !cmdPath.endsWith("/")) {
			cmdPath = cmdPath + "/";
		}
		cmdPath = cmdPath + JOB_STATUS_CMD + " " + job.getJobname() + " ";
		if (job.getJobid() != null) {
			cmdPath = cmdPath + job.getJobid();
		}
		return cmdPath;
	}
	
	/*
	 * Return the environment to use for the child process.  This is
	 * the current environment with _BPX_SHAREAS and _BPX_SPAWN_SCRIPT set to "YES"
	 * so that the child process will execute in the same address space.
	 */
	protected static String[] getEnvironment() {
		Properties p = ZUtil.getEnvironment();
		p.put("_BPX_SHAREAS", "YES");
		p.put("_BPX_SPAWN_SCRIPT", "YES");
		String[] environ = new String[p.size()];
		int i=0;
	    for (Iterator iter = p.keySet().iterator(); iter.hasNext();) {
	    	String key = (String)iter.next();
	    	environ[i++] = key + "=" + p.getProperty(key);
	    }
	    return environ;
	}	
	
	/**
	 * Copy a file given its name to a Writer.
	 * After copying or on error, close both files.
	 */
	protected static void copyFile(String filename, Writer writer) throws IOException {
		
		BufferedReader rdr = null;
		try {
			rdr = FileFactory.newBufferedReader(filename);
			String line;
			while ((line = rdr.readLine()) != null) {
				writer.write(line);
				writer.write("\n");
			}
		
		} finally {
			writer.close();
			if (rdr != null) {
				rdr.close();
			}
		}			
	}
}
